import os
import tornado.web

from tornado.escape import json_encode
import subprocess


class UpdateRequestHandler(tornado.web.RequestHandler):
    isroot = False

    def set_default_headers(self):
        self.set_header("Access-Control-Allow-Origin", "*")

    def initialize(self, core, config):
        self.core = core

        # Check if root user
        if os.geteuid() == 0:
            self.isroot = True

    def get(self):
        self.write(json_encode({'response': self.isroot}))

    def post(self):
        if not self.isroot:
            resp = 'Mopidy needs to run as root user to perform an auto-update'
        else:
            try:
                subprocess.check_call(["pip", "install", "--upgrade", "mopidy-mopify"])
                resp = 'Update succesful'
            except subprocess.CalledProcessError:
                resp = "The auto-update failed"

        self.write(json_encode({'response': resp}))


def mopify_update_factory(config, core):
    return [
        ('/', UpdateRequestHandler, {'core': core, 'config': config})
    ]



from __future__ import unicode_literals

from mopidy.core import CoreListener
import pykka

from .. import mem

class QueueManagerFrontend(pykka.ThreadingActor, CoreListener):

    def __init__(self, config, core):
        super(QueueManagerFrontend, self).__init__()
        self.config = config
        self.core = core

    def track_playback_started(self, tl_track):
        tlids = [tl_track.tlid]

        mem.queuemanager.remove_from_queue(tlids)
        mem.queuemanager.remove_from_playlist(tlids)



from __future__ import unicode_literals
import logging

import tornado.websocket

import json

from .. import mem

logger = logging.getLogger(__name__)


class RequestHandler(tornado.websocket.WebSocketHandler):

    def initialize(self, core, config, instance):
        self.core = core
        self.instance = instance

    def check_origin(self, origin):
        return True

    def open(self, args):
        logger.debug("QueueManager WebSocket opened")

    def on_message(self, message):
        if not message:
            return

        logger.debug("Message received: %s", message)

        data = tornado.escape.json_decode(message)

        if type(data['data']) is dict:
            args = data['data']
        else:
            args = {}

        call = getattr(mem.queuemanager, data['method'])(**args)

        result = {
            'call': call,
            'id': data['id']
        }

        self.write_message(json.dumps(result))

    def on_close(self):
        logger.debug("QueueManager WebSocket closed")



from __future__ import unicode_literals

import logging
import os
import tornado.web
import sync
import update 
import mem
from queuemanager import core
from queuemanager import frontend
from queuemanager import requesthandler

from mopidy import config, ext

__version__ = '1.5.5'
__ext_name__ = 'mopify'
__verbosemode__ = False

logger = logging.getLogger(__ext_name__)

class MopifyExtension(ext.Extension):
    dist_name = 'Mopidy-Mopify'
    ext_name = __ext_name__
    version = __version__

    def get_default_config(self):
        conf_file = os.path.join(os.path.dirname(__file__), 'ext.conf')
        return config.read(conf_file)

    def get_config_schema(self):
        schema = super(MopifyExtension, self).get_config_schema()
        schema['debug'] = config.Boolean()
        return schema

    def setup(self, registry):
        syncinstance = sync.Sync();
        
        mem.queuemanager = core.QueueManager()

        # Add Queuemanager Frontend class
        registry.add('frontend', frontend.QueueManagerFrontend)

        # Add web extension
        registry.add('http:app', {
            'name': self.ext_name,
            'factory': mopify_client_factory  
        })

        logger.info('Setup Mopify')

def mopify_client_factory(config, core):
    directory = 'debug' if (config.get(__ext_name__)['debug'] == True) else 'min'
    mopifypath = os.path.join(os.path.dirname(__file__), 'static', directory)

    return [
        ('/sync/(.*)', sync.RootRequestHandler, {'core': core, 'config': config}),
        ('/queuemanager/(.*)', requesthandler.RequestHandler, {'core': core, 'config': config, 'instance': mem.queuemanager}),
        ('/update', update.UpdateRequestHandler, {'core': core, 'config': config}),
        (r'/(.*)', tornado.web.StaticFileHandler, {
            "path": mopifypath,
            "default_filename": "index.html"
        })
    ]


from __future__ import unicode_literals
import os

import tornado.web

from configobj import ConfigObj


class Sync(object):
    # Define variables
    userhome = os.getenv("HOME")
    directory = ".config/mopidy-mopify"
    filename = "sync.ini"

    # build path
    syncfile = os.path.join(userhome, directory, filename)

    # check if path exists, otherwise create it
    def __init__(self):
        if not os.path.exists(os.path.join(self.userhome, self.directory)):
            os.makedirs(os.path.join(self.userhome, self.directory))


class RootRequestHandler(tornado.web.RequestHandler):
    def set_default_headers(self):
        self.set_header("Access-Control-Allow-Origin", "*")

    def initialize(self, core, config):
        self.core = core

    def get(self, service):
        response = {}

        if service == "spotify":
            spotify = Spotify()
            response = spotify.read()

        if service == "tasteprofile":
            tasteprofile = TasteProfile()
            response = tasteprofile.read()

        if service == "clients":
            clients = Clients()
            response = clients.read()

        if service == "settings":
            settings = Settings()
            response = settings.read()

        self.write({"response": response})

    def post(self, service):
        response = {}

        if service == "spotify":
            spotify = Spotify()
            response = spotify.write({
                'refresh_token': self.get_argument('refresh_token', default=''),
                'access_token': self.get_argument('access_token', default=''),
                'client_id': self.get_argument('client_id', default='')
            })

        if service == "tasteprofile":
            tasteprofile = TasteProfile()
            response = tasteprofile.write({
                'id': self.get_argument('id', default=''),
                'client_id': self.get_argument('client_id', default='')
            })

        if service == "clients":
            clients = Clients()
            response = clients.write({
                'name': self.get_argument("name", default=''),
                'client_id': self.get_argument("client_id", default=''),
                'master': self.get_argument("master", default=False)
            })

        if service == "settings":
            settings = Settings()
            response = settings.write({
                'forcesync': self.get_argument("forcesync", default='')
            })

        self.write({"response": response})


class Spotify(Sync):

    def __init__(self):
        self.syncini = ConfigObj(self.syncfile, encoding='UTF8', create_empty=True)

        # Check if the key exists
        try:
            self.syncini["spotify"]
        except KeyError:
            self.syncini["spotify"] = {}

    def read(self):
        try:
            spotify = self.syncini['spotify']
            spotify['client'] = self.syncini['accounts'][spotify['client_id']]
        except KeyError:
            resp = "No data available"

        resp = spotify

        return resp

    def write(self, arguments):
        # Create the section
        self.syncini['spotify'] = arguments

        # Write to ini file
        self.syncini.write()

        return self.syncini['spotify']


class TasteProfile(Sync):

    def __init__(self):
        self.syncini = ConfigObj(self.syncfile, encoding='UTF8', create_empty=True)

        # Check if the key exists
        try:
            self.syncini["tasteprofile"]
        except KeyError:
            self.syncini["tasteprofile"] = {}

    def read(self):
        try:
            tasteprofile = self.syncini['tasteprofile']
            tasteprofile['client'] = self.syncini['accounts'][tasteprofile['client_id']]

            resp = tasteprofile
        except KeyError:
            resp = "No data available"

        return resp

    def write(self, arguments):
        # Create the section
        self.syncini['tasteprofile'] = arguments

        # Write to ini file
        self.syncini.write()
        return self.syncini['tasteprofile']


class Clients(Sync):

    def __init__(self):
        self.syncini = ConfigObj(self.syncfile, encoding='UTF8', create_empty=True)

        # Check if the key exists
        try:
            self.syncini["accounts"]
        except KeyError:
            self.syncini["accounts"] = {}

    def read(self):
        return self.syncini["accounts"]

    def write(self, arguments):
        accounts = self.syncini["accounts"]

        client = accounts.get(arguments["client_id"])

        if not client:
            accounts[arguments["client_id"]] = arguments
            self.syncini.write()

            resp = self.syncini["accounts"]
        else:
            client["name"] = arguments["name"]

            self.syncini.write()

            resp = client

        # Write to ini file
        self.syncini.write()
        return resp


class Settings(Sync):

    def __init__(self):
        self.syncini = ConfigObj(self.syncfile, encoding='UTF8', create_empty=True)

        # Check if the key exists
        try:
            self.syncini["settings"]
        except KeyError:
            self.syncini["settings"] = {}

    def read(self):
        try:
            settings = self.syncini['settings']

            resp = settings
        except KeyError:
            resp = "No data available"

        return resp

    def write(self, arguments):
        # Create the section
        self.syncini['settings'] = arguments

        # Write to ini file
        self.syncini.write()
        return self.syncini['settings']



from __future__ import unicode_literals

import re
from setuptools import setup, find_packages


def get_version(filename):
    content = open(filename).read()
    metadata = dict(re.findall("__([a-z]+)__ = '([^']+)'", content))
    return metadata['version']


setup(
    name='Mopidy-Mopify',
    version=get_version('mopidy_mopify/__init__.py'),
    url='https://github.com/dirkgroenen/Mopify',
    license='GNU General Public License v3 (GPLv3)',
    author='Dirk Groenen',
    author_email='dirk@bitlabs.nl',
    description='A Mopidy Web client based on the (old) Spotify interface. Improved to work with spotify as main library.',
    long_description=open('README.md').read(),
    packages=find_packages(),
    zip_safe=False,
    include_package_data=True,
    install_requires=[
        'setuptools',
        'Mopidy >= 0.19',
        'ConfigObj'
    ],
    entry_points={
        'mopidy.ext': [
            'mopify = mopidy_mopify:MopifyExtension',
        ],
    },
    classifiers=[
        'Environment :: No Input/Output (Daemon)',
        'Intended Audience :: End Users/Desktop',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 2',
        'Topic :: Multimedia :: Sound/Audio :: Players',
    ],
)


from __future__ import unicode_literals


class QueueManager(object):
    # Initialize the tracklist
    queue = []
    playlist = []
    shufflememory = []

    shuffled = False
    version = 0

    def get_all(self):
        return {
            'queue': self.queue,
            'playlist': self.playlist,
            'shuffle': self.shuffled,
            'version': self.version
        }

    def get_shuffle(self):
        return self.shuffled

    def add_to_queue(self, tracks):
        self.queue.extend(tracks)
        self.version += 1

        return {
            'queue': self.queue,
            'version': self.version
        }

    def add_play_next(self, tracks):
        self.queue.insert(0, tracks[0])
        self.version += 1

        return {
            'queue': self.queue,
            'version': self.version
        }

    def remove_from_queue(self, tlids):
        self.queue = [tltrack for tltrack in self.queue if tltrack["tlid"] not in tlids]
        self.shufflememory = [tltrack for tltrack in self.shufflememory if tltrack["tlid"] not in tlids]

        self.version += 1

        return {
            'queue': self.queue,
            'version': self.version
        }

    def remove_from_playlist(self, tlids):
        self.playlist = [tltrack for tltrack in self.playlist if tltrack["tlid"] not in tlids]
        self.shufflememory = [tltrack for tltrack in self.shufflememory if tltrack["tlid"] not in tlids]

        self.version += 1

        return {
            'playlist': self.playlist,
            'version': self.version
        }

    def remove_from_tracklist(self, tlids):
        self.remove_from_playlist(tlids)
        self.remove_from_queue(tlids)

        return self.get_all()

    def clear_queue(self, tracks):
        self.queue = []
        self.version += 1

        return {
            'queue': self.queue,
            'version': self.version
        }

    def set_playlist(self, tracks):
        self.playlist = tracks
        self.version += 1

        return {
            'playlist': self.playlist,
            'version': self.version
        }

    def shuffle_playlist(self, tracks):
        self.shufflememory = self.playlist
        self.playlist = tracks
        self.shuffled = True
        self.version += 1

        return self.get_all()

    def shuffle_reset(self):
        self.shuffled = False
        self.playlist = self.shufflememory
        self.shufflememory = []
        self.version += 1

        return self.get_all()

    def replace_all(self, playlist=[], queue=[]):
        self.version += 1
        self.playlist = playlist
        self.queue = queue

        return self.get_all()
